@@ -1,20 +1,20 @@
 /***************************************************************
 *
-*  Evaluation of Typo3 form field content
+*  Evaluation of TYPO3 form field content
 *
-* $Id: jsfunc.evalfield.js 1421 2006-04-10 09:27:15Z mundaun $
+* $Id: jsfunc.evalfield.js 3554 2008-04-11 15:43:07Z baschny $
 *
 *
 *
 *  Copyright notice
 *
-*  (c) 1998-2003 Kasper Skaarhoj
+*  (c) 1998-2008 Kasper Skaarhoj
 *  All rights reserved
 *
 *  This script is part of the TYPO3 t3lib/ library provided by
 *  Kasper Skaarhoj <kasper@typo3.com> together with TYPO3
 *
-*  Released under GNU/GPL (see license file in tslib/)
+*  Released under GNU/GPL (see license file in typo3/sysext/cms/tslib/)
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -34,6 +34,9 @@ function evalFunc()	{
 	this.getSecs = evalFunc_getSecs;
 	this.getYear = evalFunc_getYear;
 	this.getTimeSecs = evalFunc_getTimeSecs;
+	this.getTime = evalFunc_getTime;
+	this.getDate = evalFunc_getDate;
+	this.getTimestamp = evalFunc_getTimestamp;
 	this.caseSwitch = evalFunc_caseSwitch;
 	this.evalObjValue = evalFunc_evalObjValue;
 	this.outputObjValue = evalFunc_outputObjValue;
@@ -44,8 +47,9 @@ function evalFunc()	{
 	this.btrim = evalFunc_btrim;
 	var today = new Date();
  	this.lastYear = this.getYear(today);
- 	this.lastDate = this.getSecs(today);
- 	this.lastTime = this.getTimeSecs(today);
+ 	this.lastDate = this.getDate(today);
+ 	this.lastTime = this.getTimestamp(today);
+	this.refDate = today;
 	this.isInString = '';
 	this.USmode = 0;
 }
@@ -77,7 +81,9 @@ function evalFunc_outputObjValue(FObj,value)	{
 	var theEvalType = this.split(evallist, ",", index);
 	var newValue=value;
 	while (theEvalType) {
-		newValue = evalFunc.output(theEvalType, value, FObj);
+		if (theEvalType != 'required') {
+			newValue = evalFunc.output(theEvalType, value, FObj);
+		}
 		index++;
 		theEvalType = this.split(evallist, ",", index);
 	}
@@ -245,7 +251,7 @@ function evalFunc_input(type,inVal)	{
 		return this.parseDouble(inVal);
 	}
 
-	var today = new Date()
+	var today = new Date();
 	var add=0;
 	var value = this.ltrim(inVal);
 	var values = new evalFunc_split(value);
@@ -261,8 +267,6 @@ function evalFunc_input(type,inVal)	{
 				case "d":
 				case "t":
 				case "n":
-					var theTime = new Date(this.getYear(today), today.getMonth(), today.getDate(), today.getHours(), today.getMinutes());
-					this.lastDate = this.getSecs(theTime)
 					if (values.valPol[1])	{
 						add = this.pol(values.valPol[1],this.parseInt(values.values[1]));
 					}
@@ -276,20 +280,20 @@ function evalFunc_input(type,inVal)	{
 				default:
 					var index = value.indexOf(' ');
 					if (index!=-1)	{
-						var theSecs = this.input("date",value.substr(index,value.length)) + this.input("time",value.substr(0,index));
-						this.lastDate = theSecs;
+						var dateVal = this.input("date",value.substr(index,value.length));
+							// set refDate so that evalFunc_input on time will work with correct DST information
+						this.refDate = new Date(dateVal*1000);
+						this.lastTime = dateVal + this.input("time",value.substr(0,index));
 					}
 			}
-			this.lastDate+=add*24*60*60;
-			return this.lastDate;
+			this.lastTime+=add*24*60*60;
+			return this.lastTime;
 		break;
 		case "year":
 			switch (theCmd)	{
 				case "d":
 				case "t":
 				case "n":
-					var theTime = today;
-					this.lastYear = this.getYear(theTime);
 					if (values.valPol[1])	{
 						add = this.pol(values.valPol[1],this.parseInt(values.values[1]));
 					}
@@ -305,11 +309,13 @@ function evalFunc_input(type,inVal)	{
 						add = this.pol(values.valPol[2],this.parseInt(values.values[2]));
 					}
 					var year = (values.values[1])?this.parseInt(values.values[1]):this.getYear(today);
-						if (  (year>=0&&year<38) || (year>=70&&year<100) || (year>=1970&&year<2038)	)	{
-							if (year<100)	{
-								year = (year<38) ? year+=2000 : year+=1900;
-							}
-						} else {year = this.getYear(today);}
+					if (  (year>=0&&year<38) || (year>=70&&year<100) || (year>=1902&&year<2038)	)	{
+						if (year<100)	{
+							year = (year<38) ? year+=2000 : year+=1900;
+						}
+					} else {
+						year = this.getYear(today);
+					}
 					this.lastYear = year
 			}
 			this.lastYear+=add;
@@ -320,8 +326,6 @@ function evalFunc_input(type,inVal)	{
 				case "d":
 				case "t":
 				case "n":
-					var theTime = new Date(this.getYear(today), today.getMonth(), today.getDate());
-					this.lastDate = this.getSecs(theTime);
 					if (values.valPol[1])	{
 						add = this.pol(values.valPol[1],this.parseInt(values.values[1]));
 					}
@@ -346,26 +350,24 @@ function evalFunc_input(type,inVal)	{
 					}
 
 					var year = (values.values[3])?this.parseInt(values.values[3]):this.getYear(today);
-						if (  (year>=0&&year<38) || (year>=70&&year<100) || (year>=1970&&year<2038)	)	{
-							if (year<100)	{
-								year = (year<38) ? year+=2000 : year+=1900;
-							}
-						} else {year = this.getYear(today);}
-					var month = (values.values[this.USmode?1:2])?this.parseInt(values.values[this.USmode?1:2]):today.getMonth()+1;
-						if (month > 12)	{month=12;}
-						if (month < 1)	{month=1;}
-					var day = (values.values[this.USmode?2:1])?this.parseInt(values.values[this.USmode?2:1]):today.getDate();
-						if (day > 31)	{day=31;}
-						if (day < 1)	{day=1;}
-					if (''+day+'-'+month+'-'+year == "1-1-1970")	{
-						var theTime = new Date();  theTime.setTime(0);
+					if ( (year>=0&&year<38) || (year>=70&&year<100) || (year>=1902&&year<2038) )	{
+						if (year<100)	{
+							year = (year<38) ? year+=2000 : year+=1900;
+						}
 					} else {
-						var theTime = new Date(parseInt(year), parseInt(month)-1, parseInt(day));
+						year = this.getYear(today);
 					}
-					this.lastDate = this.getSecs(theTime)
+					var month = (values.values[this.USmode?1:2])?this.parseInt(values.values[this.USmode?1:2]):today.getUTCMonth()+1;
+					var day = (values.values[this.USmode?2:1])?this.parseInt(values.values[this.USmode?2:1]):today.getUTCDate();
+
+					var theTime = new Date(parseInt(year), parseInt(month)-1, parseInt(day));
+
+						// Substract timezone offset from client
+					this.lastDate = this.getTimestamp(theTime);
+					theTime.setTime((this.lastDate - theTime.getTimezoneOffset()*60)*1000);
+					this.lastDate = this.getTimestamp(theTime);
 			}
 			this.lastDate+=add*24*60*60;
-			if (this.lastDate<0) {this.lastDate=0;}
 			return this.lastDate;
 		break;
 		case "time":
@@ -374,8 +376,6 @@ function evalFunc_input(type,inVal)	{
 				case "d":
 				case "t":
 				case "n":
-					var theTime = new Date(this.getYear(today), today.getMonth(), today.getDate(), today.getHours(), today.getMinutes(), ((type=="timesec")?today.getSeconds():0));
-					this.lastTime = this.getTimeSecs(theTime);
 					if (values.valPol[1])	{
 						add = this.pol(values.valPol[1],this.parseInt(values.values[1]));
 					}
@@ -398,14 +398,18 @@ function evalFunc_input(type,inVal)	{
 						var temp = values.values[1];
 						values = new evalFunc_splitSingle(temp);
 					}
-					var sec = (values.values[3])?this.parseInt(values.values[3]):today.getSeconds();
-						if (sec > 59)	{sec=59;}
-					var min = (values.values[2])?this.parseInt(values.values[2]):today.getMinutes();
-						if (min > 59)	{min=59;}
-					var hour = (values.values[1])?this.parseInt(values.values[1]):today.getHours();
-						if (hour > 23)	{hour=23;}
-					var theTime = new Date(this.getYear(today), today.getMonth(), today.getDate(), hour, min, ((type=="timesec")?sec:0));
-					this.lastTime = this.getTimeSecs(theTime)
+					var sec = (values.values[3])?this.parseInt(values.values[3]):today.getUTCSeconds();
+					if (sec > 59)	{sec=59;}
+					var min = (values.values[2])?this.parseInt(values.values[2]):today.getUTCMinutes();
+					if (min > 59)	{min=59;}
+					var hour = (values.values[1])?this.parseInt(values.values[1]):today.getUTCHours();
+					if (hour > 23)	{hour=23;}
+
+					var theTime = new Date(this.getYear(this.refDate), this.refDate.getUTCMonth(), this.refDate.getUTCDate(), hour, min, ((type=="timesec")?sec:0));
+
+					this.lastTime = this.getTimestamp(theTime);
+					theTime.setTime((this.lastTime - theTime.getTimezoneOffset()*60)*1000);
+					this.lastTime = this.getTime(theTime);
 			}
 			this.lastTime+=add*60;
 			if (this.lastTime<0) {this.lastTime+=24*60*60;}
@@ -420,28 +424,24 @@ function evalFunc_output(type,value,FObj)	{
 	switch (type)	{
 		case "date":
 			if (!parseInt(value))	{return '';}
-			var theTime = new Date();
-			theTime.setTime(value*1000);
+			var theTime = new Date(parseInt(value) * 1000);
 			if (this.USmode)	{
-				theString = (theTime.getMonth()+1)+'-'+theTime.getDate()+'-'+this.getYear(theTime);
+				theString = (theTime.getUTCMonth()+1)+'-'+theTime.getUTCDate()+'-'+this.getYear(theTime);
 			} else {
-				theString = theTime.getDate()+'-'+(theTime.getMonth()+1)+'-'+this.getYear(theTime);
+				theString = theTime.getUTCDate()+'-'+(theTime.getUTCMonth()+1)+'-'+this.getYear(theTime);
 			}
 		break;
 		case "datetime":
 			if (!parseInt(value))	{return '';}
-			var theTime = new Date();
-			theTime.setTime(value*1000);
-			theString = this.output("time",this.getTimeSecs(theTime))+' '+this.output("date",value);
+			theString = this.output("time",value)+' '+this.output("date",value);
 		break;
 		case "time":
 		case "timesec":
 			if (!parseInt(value))	{return '';}
-			var theTime = new Date();
-			theTime.setTime(value*1000);
-			var h = Math.floor(value/3600);
-			var m = Math.floor((value-h*3600)/60);
-			var s = Math.floor(value-h*3600-m*60);
+			var theTime = new Date(parseInt(value) * 1000);
+			var h = theTime.getUTCHours();
+			var m = theTime.getUTCMinutes();
+			var s = theTime.getUTCSeconds();
 			theString = h+':'+((m<10)?'0':'')+m + ((type=="timesec")?':'+((s<10)?'0':'')+s:'');
 		break;
 		case "password":
@@ -456,14 +456,23 @@ function evalFunc_output(type,value,FObj)	{
 	return theString;
 }
 function evalFunc_getSecs(timeObj)	{
-	return Math.round(timeObj.getTime()/1000);
+	return Math.round(timeObj.getUTCSeconds()/1000);
+}
+// Seconds since midnight:
+function evalFunc_getTime(timeObj)	{
+	return timeObj.getUTCHours()*60*60+timeObj.getUTCMinutes()*60+Math.round(timeObj.getUTCSeconds()/1000);
 }
 function evalFunc_getYear(timeObj)	{
-	return (timeObj.getYear()>200) ? timeObj.getYear() : (timeObj.getYear()+1900);
+	return timeObj.getUTCFullYear();
 }
+// Seconds since midnight with client timezone offset:
 function evalFunc_getTimeSecs(timeObj)	{
 	return timeObj.getHours()*60*60+timeObj.getMinutes()*60+timeObj.getSeconds();
 }
+function evalFunc_getDate(timeObj)	{
+	var theTime = new Date(this.getYear(timeObj), timeObj.getUTCMonth(), timeObj.getUTCDate());
+	return this.getTimestamp(theTime);
+}
 function evalFunc_dummy (evallist,is_in,checkbox,checkboxValue) {
 	this.evallist = evallist;
 	this.is_in = is_in;
@@ -483,4 +492,6 @@ function evalFunc_splitStr(theStr1, delim, index) {
 	if(ePos == -1)	{ePos = theStr.length;}
 	return (theStr.substring(sPos+lengthOfDelim,ePos));
 }
-
+function evalFunc_getTimestamp(timeObj)	{
+	return Date.parse(timeObj)/1000;
+}
