@@ -2,7 +2,7 @@
 /***************************************************************
 *  Copyright notice
 *
-*  (c) 1999-2005 Kasper Skaarhoj (kasperYYYY@typo3.com)
+*  (c) 1999-2008 Kasper Skaarhoj (kasperYYYY@typo3.com)
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
@@ -30,7 +30,7 @@
  * Provides links to registered shortcuts
  * If the 'cms' extension is loaded you will also have a field for entering page id/alias which will be found/edited
  *
- * $Id: alt_shortcut.php 1421 2006-04-10 09:27:15Z mundaun $
+ * $Id: alt_shortcut.php 3510 2008-04-01 20:42:57Z flyguide $
  * Revised for TYPO3 3.6 2/2003 by Kasper Skaarhoj
  * XHTML compliant output
  *
@@ -96,12 +96,24 @@ class SC_alt_shortcut {
 	var $whichItem;
 
 		// Internal, static:
-	var $loadModules;		// Modules object
-	var $doc;			// Document template object
+	/**
+	 * Object for backend modules, load modules-object
+	 *
+	 * @var t3lib_loadModules
+	 */
+	var $loadModules;
+	protected $isAjaxCall;
+
+	/**
+	 * Document template object
+	 *
+	 * @var template
+	 */
+	var $doc;
 
 		// Internal, dynamic:
 	var $content;			// Accumulation of output HTML (string)
-	var $lines;			// Accumulation of table cells (array)
+	var $lines;				// Accumulation of table cells (array)
 
 	var $editLoaded;		// Flag for defining whether we are editing
 	var $editError;			// Can contain edit error message
@@ -126,18 +138,19 @@ class SC_alt_shortcut {
 		global $TBE_MODULES;
 
 			// Setting GPvars:
-		$this->modName = t3lib_div::_GP('modName');
-		$this->M_modName = t3lib_div::_GP('motherModName');
-		$this->URL = t3lib_div::_GP('URL');
-		$this->editSC = t3lib_div::_GP('editShortcut');
+		$this->isAjaxCall             = (boolean) t3lib_div::_GP('ajax');
+		$this->modName                = t3lib_div::_GP('modName');
+		$this->M_modName              = t3lib_div::_GP('motherModName');
+		$this->URL                    = t3lib_div::_GP('URL');
+		$this->editSC                 = t3lib_div::_GP('editShortcut');
 
-		$this->deleteCategory = t3lib_div::_GP('deleteCategory');
-		$this->editPage = t3lib_div::_GP('editPage');
-		$this->changeWorkspace = t3lib_div::_GP('changeWorkspace');
+		$this->deleteCategory         = t3lib_div::_GP('deleteCategory');
+		$this->editPage               = t3lib_div::_GP('editPage');
+		$this->changeWorkspace        = t3lib_div::_GP('changeWorkspace');
 		$this->changeWorkspacePreview = t3lib_div::_GP('changeWorkspacePreview');
-		$this->editName = t3lib_div::_GP('editName');
-		$this->editGroup = t3lib_div::_GP('editGroup');
-		$this->whichItem = t3lib_div::_GP('whichItem');
+		$this->editName               = t3lib_div::_GP('editName');
+		$this->editGroup              = t3lib_div::_GP('editGroup');
+		$this->whichItem              = t3lib_div::_GP('whichItem');
 
 			// Creating modules object
 		$this->loadModules = t3lib_div::makeInstance('t3lib_loadModules');
@@ -262,6 +275,9 @@ class SC_alt_shortcut {
 			function changeWorkspacePreview(newstate)	{	//
 				window.location.href="alt_shortcut.php?changeWorkspacePreview="+newstate;
 			}
+			function refreshShortcuts() {
+				window.location.href = document.URL;
+			}
 
 			');
 		$this->content.=$this->doc->startPage('Shortcut frame');
@@ -394,7 +410,7 @@ class SC_alt_shortcut {
 		if (!$this->editLoaded && t3lib_extMgm::isLoaded('cms'))	{
 				$editIdCode = '<td nowrap="nowrap">'.$LANG->getLL('shortcut_editID',1).': <input type="text" value="'.($this->editError?htmlspecialchars($this->editPage):'').'" name="editPage"'.$this->doc->formWidth(15).' onchange="submitEditPage(this.value);" />'.
 					($this->editError?'&nbsp;<strong><span class="typo3-red">'.htmlspecialchars($this->editError).'</span></strong>':'').
-					(is_array($this->theEditRec)?'&nbsp;<strong>'.$LANG->getLL('shortcut_loadEdit',1).' \''.t3lib_BEfunc::getRecordTitle('pages',$this->theEditRec,1).'\'</strong> ('.htmlspecialchars($this->editPath).')':'').
+					(is_array($this->theEditRec)?'&nbsp;<strong>'.$LANG->getLL('shortcut_loadEdit',1).' \''.t3lib_BEfunc::getRecordTitle('pages',$this->theEditRec,TRUE).'\'</strong> ('.htmlspecialchars($this->editPath).')':'').
 					($this->searchFor?'&nbsp;'.$LANG->getLL('shortcut_searchFor',1).' <strong>\''.htmlspecialchars($this->searchFor).'\'</strong>':'').
 					'</td>';
 		} else $editIdCode = '';
@@ -418,9 +434,12 @@ class SC_alt_shortcut {
 							</tr>
 						</table>
 					</td>
-					<td align="right">
-						'.$this->workspaceSelector().
-							t3lib_BEfunc::cshItem('xMOD_csh_corebe', 'workspaceSelector', $GLOBALS['BACK_PATH'],'',TRUE).'
+					<td align="right">';
+		if ($this->hasWorkspaceAccess()) {
+			$this->content .= $this->workspaceSelector() .
+								t3lib_BEfunc::cshItem('xMOD_csh_corebe', 'workspaceSelector', $GLOBALS['BACK_PATH'],'',TRUE);
+		}
+		$this->content .= '
 					</td>
 				</tr>
 			</table>
@@ -429,6 +448,7 @@ class SC_alt_shortcut {
 			// Launch Edit page:
 		if ($this->theEditRec['uid'])	{
 			$this->content.=$this->doc->wrapScriptTags('top.loadEditId('.$this->theEditRec['uid'].');');
+
 		}
 
 			// Load alternative table/uid into editing form.
@@ -516,7 +536,7 @@ class SC_alt_shortcut {
 
 		if (count($this->lines))	{
 			if (!$BE_USER->getTSConfigVal('options.mayNotCreateEditShortcuts'))	{
-				$this->lines=array_merge(array('<td><input type="checkbox" id="editShortcut_check" name="editShortcut_check" value="1"'.($this->editSC?' checked="checked"':'').' /><label for="editShortcut_check">'.$LANG->getLL('shortcut_edit',1).'</label>&nbsp;</td>'),$this->lines);
+				$this->lines=array_merge(array('<td><input type="checkbox" id="editShortcut_check" name="editShortcut_check" value="1"'.($this->editSC?' checked="checked"':'').' /> <label for="editShortcut_check">'.$LANG->getLL('shortcut_edit',1).'</label>&nbsp;</td>'),$this->lines);
 				$this->lines[]='<td>'.$manageForm.'</td>';
 			}
 			$this->lines[]='<td><img src="clear.gif" width="10" height="1" alt="" /></td>';
@@ -535,7 +555,7 @@ class SC_alt_shortcut {
 		if (!t3lib_extMgm::isLoaded('cms'))	return;
 
 			// EDIT page:
-		$this->editPage = trim(strtolower($this->editPage));
+		$this->editPage = trim($LANG->csConvObj->conv_case($LANG->charSet,$this->editPage,'toLower'));
 		$this->editError = '';
 		$this->theEditRec = '';
 		$this->searchFor = '';
@@ -584,9 +604,47 @@ class SC_alt_shortcut {
 	 * @return	void
 	 */
 	function printContent()	{
+		$content = '';
+
 		$this->content.= $this->doc->endPage();
 		$this->content = $this->doc->insertStylesAndJS($this->content);
-		echo $this->content;
+
+		if($this->editPage && $this->isAjaxCall) {
+			require_once('contrib/json/json.php');
+			$data = array();
+
+				// edit page
+			if($this->theEditRec['uid']) {
+				$data['type']       = 'page';
+				$data['editRecord'] = $this->theEditRec['uid'];
+			}
+
+				// edit alternative table/uid
+			if(count($this->alternativeTableUid) == 2
+			&& isset($GLOBALS['TCA'][$this->alternativeTableUid[0]])
+			&& t3lib_div::testInt($this->alternativeTableUid[1])) {
+				$data['type']             = 'alternative';
+				$data['alternativeTable'] = $this->alternativeTableUid[0];
+				$data['alternativeUid']   = $this->alternativeTableUid[1];
+			}
+
+				// search for something else
+			if($this->searchFor) {
+				$data['type']            = 'search';
+				$data['firstMountPoint'] = intval($GLOBALS['WEBMOUNTS'][0]);
+				$data['searchFor']       = rawurlencode($this->searchFor);
+			}
+
+			$json = new Services_JSON();
+			$content = $json->encode($data);
+
+			header('Content-type: application/json; charset=utf-8');
+			header('X-JSON: '.$content);
+		} else {
+			$content = $this->content;
+		}
+
+		echo $content;
 	}
 
 
@@ -614,7 +672,7 @@ class SC_alt_shortcut {
 			// Changing workspace and if so, reloading entire backend:
 		if (strlen($this->changeWorkspace))	{
 			$BE_USER->setWorkspace($this->changeWorkspace);
-			return $this->doc->wrapScriptTags('top.location.href="alt_main.php";');
+			return $this->doc->wrapScriptTags('top.location.href="'. t3lib_BEfunc::getBackendScript() . '";');
 		}
 			// Changing workspace and if so, reloading entire backend:
 		if (strlen($this->changeWorkspacePreview))	{
@@ -653,13 +711,15 @@ class SC_alt_shortcut {
 		$selector = '';
 			// Preview:
 		if ($BE_USER->workspace!==0)	{
-			$selector.= 'Frontend Preview: <input type="checkbox" onclick="changeWorkspacePreview('.($BE_USER->user['workspace_preview'] ? 0 : 1).')"; '.($BE_USER->user['workspace_preview'] ? 'checked="checked"' : '').'/>&nbsp;';
+			$selector.= '<label for="workspacePreview">Frontend Preview:</label> <input type="checkbox" name="workspacePreview" id="workspacePreview" onclick="changeWorkspacePreview('.($BE_USER->user['workspace_preview'] ? 0 : 1).')"; '.($BE_USER->user['workspace_preview'] ? 'checked="checked"' : '').'/>&nbsp;';
 		}
 
 		$selector.= '<a href="mod/user/ws/index.php" target="content">'.
 					t3lib_iconWorks::getIconImage('sys_workspace',array(),$this->doc->backPath,'align="top"').
-					'</a>'.
-					'<select name="_workspaceSelector" onchange="changeWorkspace(this.options[this.selectedIndex].value);">'.implode('',$options).'</select>';
+					'</a>';
+		if (count($options) > 1) {
+			$selector .= '<select name="_workspaceSelector" onchange="changeWorkspace(this.options[this.selectedIndex].value);">'.implode('',$options).'</select>';
+		}
 
 		return $selector;
 	}
@@ -748,6 +808,17 @@ class SC_alt_shortcut {
 	function getLinkedPageId($url)	{
 		return preg_replace('/.*[\?&]id=([^&]+).*/', '$1', $url);
 	}
+
+	/**
+	 * Checks if user has access to Workspace module.
+	 *
+	 * @return	boolean		Returns true if user has access to workspace module.
+	 */
+	function hasWorkspaceAccess() {
+		$MCONF = array();
+		include('mod/user/ws/conf.php');
+		return $GLOBALS['BE_USER']->modAccess(array('name' => 'user', 'access' => 'user,group'), false) && $GLOBALS['BE_USER']->modAccess($MCONF, false);
+	}
 }
 
 // Include extension?
