@@ -2,7 +2,7 @@
 /***************************************************************
 *  Copyright notice
 *
-*  (c) 1999-2004 Kasper Skaarhoj (kasperYYYY@typo3.com)
+*  (c) 1999-2008 Kasper Skaarhoj (kasperYYYY@typo3.com)
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
@@ -28,7 +28,7 @@
  * Dynamic configuation of the tt_content table
  * This gets it's own file because it's so huge and central to typical TYPO3 use.
  *
- * $Id: tbl_tt_content.php,v 1.17 2004/09/13 22:57:35 typo3 Exp $
+ * $Id: tbl_tt_content.php 3793 2008-06-11 06:30:14Z ingmars $
  *
  * @author	Kasper Skaarhoj <kasperYYYY@typo3.com>
  */
@@ -47,28 +47,34 @@ $TCA['tt_content'] = Array (
 			'config' => Array (
 				'type' => 'select',
 				'items' => Array (
-					Array('LLL:EXT:cms/locallang_ttc.php:CType.I.0', 'header'),
-					Array('LLL:EXT:cms/locallang_ttc.php:CType.I.1', 'text'),
-					Array('LLL:EXT:cms/locallang_ttc.php:CType.I.2', 'textpic'),
-					Array('LLL:EXT:cms/locallang_ttc.php:CType.I.3', 'image'),
-					Array('LLL:EXT:cms/locallang_ttc.php:CType.I.4', 'bullets'),
-					Array('LLL:EXT:cms/locallang_ttc.php:CType.I.5', 'table'),
-					Array('LLL:EXT:cms/locallang_ttc.php:CType.I.6', 'uploads'),
-					Array('LLL:EXT:cms/locallang_ttc.php:CType.I.7', 'multimedia'),
-					Array('LLL:EXT:cms/locallang_ttc.php:CType.I.8', 'mailform'),
-					Array('LLL:EXT:cms/locallang_ttc.php:CType.I.9', 'search'),
-					Array('LLL:EXT:cms/locallang_ttc.php:CType.I.10', 'login'),
-					Array('LLL:EXT:cms/locallang_ttc.php:CType.I.11', 'splash'),
-					Array('LLL:EXT:cms/locallang_ttc.php:CType.I.12', 'menu'),
-					Array('LLL:EXT:cms/locallang_ttc.php:CType.I.13', 'shortcut'),
-					Array('LLL:EXT:cms/locallang_ttc.php:CType.I.14', 'list'),
-					Array('LLL:EXT:cms/locallang_ttc.php:CType.I.15', 'script'),
-					Array('LLL:EXT:cms/locallang_ttc.php:CType.I.16', 'div'),
-					Array('LLL:EXT:cms/locallang_ttc.php:CType.I.17', 'html')
+					array('LLL:EXT:cms/locallang_ttc.xml:CType.div.standard', '--div--'),
+					array('LLL:EXT:cms/locallang_ttc.xml:CType.I.0', 'header', 'i/tt_content_header.gif'),
+					array('LLL:EXT:cms/locallang_ttc.xml:CType.I.1', 'text', 'i/tt_content.gif'),
+					array('LLL:EXT:cms/locallang_ttc.xml:CType.I.2', 'textpic', 'i/tt_content_textpic.gif'),
+					array('LLL:EXT:cms/locallang_ttc.xml:CType.I.3', 'image', 'i/tt_content_image.gif'),
+					array('LLL:EXT:cms/locallang_ttc.xml:CType.div.lists', '--div--'),
+					array('LLL:EXT:cms/locallang_ttc.xml:CType.I.4', 'bullets', 'i/tt_content_bullets.gif'),
+					array('LLL:EXT:cms/locallang_ttc.xml:CType.I.5', 'table', 'i/tt_content_table.gif'),
+					array('LLL:EXT:cms/locallang_ttc.xml:CType.I.6', 'uploads', 'i/tt_content_uploads.gif'),
+					array('LLL:EXT:cms/locallang_ttc.xml:CType.div.forms', '--div--'),
+					array('LLL:EXT:cms/locallang_ttc.xml:CType.I.8', 'mailform', 'i/tt_content_form.gif'),
+					array('LLL:EXT:cms/locallang_ttc.xml:CType.I.9', 'search', 'i/tt_content_search.gif'),
+					array('LLL:EXT:cms/locallang_ttc.xml:CType.I.10', 'login', 'i/tt_content_login.gif'),
+					array('LLL:EXT:cms/locallang_ttc.xml:CType.div.special', '--div--'),
+					array('LLL:EXT:cms/locallang_ttc.xml:CType.I.7', 'multimedia', 'i/tt_content_mm.gif'),
+					array('LLL:EXT:cms/locallang_ttc.xml:CType.I.11', 'splash', 'i/tt_content_news.gif'),
+					array('LLL:EXT:cms/locallang_ttc.xml:CType.I.12', 'menu', 'i/tt_content_menu.gif'),
+					array('LLL:EXT:cms/locallang_ttc.xml:CType.I.13', 'shortcut', 'i/tt_content_shortcut.gif'),
+					array('LLL:EXT:cms/locallang_ttc.xml:CType.I.14', 'list', 'i/tt_content_list.gif'),
+					array('LLL:EXT:cms/locallang_ttc.xml:CType.I.15', 'script', 'i/tt_content_script.gif'),
+					array('LLL:EXT:cms/locallang_ttc.xml:CType.I.16', 'div', 'i/tt_content_div.gif'),
+					array('LLL:EXT:cms/locallang_ttc.xml:CType.I.17', 'html', 'i/tt_content_html.gif')
 				),
 				'default' => 'text',
 				'authMode' => $GLOBALS['TYPO3_CONF_VARS']['BE']['explicitADmode'],
 				'authMode_enforce' => 'strict',
+				'iconsInOptionTags' => 1,
+				'noIconsBelowSelect' => 1,
 			)
 		),
 		'hidden' => Array (
@@ -102,7 +108,6 @@ $TCA['tt_content'] = Array (
 				'default' => '0',
 				'range' => Array (
 					'upper' => mktime(0,0,0,12,31,2020),
-					'lower' => mktime(0,0,0,date('m')-1,date('d'),date('Y'))
 				)
 			)
 		),
@@ -111,12 +116,14 @@ $TCA['tt_content'] = Array (
 			'label' => 'LLL:EXT:lang/locallang_general.php:LGL.fe_group',
 			'config' => Array (
 				'type' => 'select',
+				'size' => 5,
+				'maxitems' => 20,
 				'items' => Array (
-					Array('', 0),
 					Array('LLL:EXT:lang/locallang_general.php:LGL.hide_at_login', -1),
 					Array('LLL:EXT:lang/locallang_general.php:LGL.any_login', -2),
 					Array('LLL:EXT:lang/locallang_general.php:LGL.usergroups', '--div--')
 				),
+				'exclusiveKeys' => '-1,-2',
 				'foreign_table' => 'fe_groups'
 			)
 		),
@@ -188,6 +195,7 @@ $TCA['tt_content'] = Array (
 		),
 		'header' => Array (
 			'l10n_mode' => 'prefixLangTitle',
+			'l10n_cat' => 'text',
 			'label' => 'LLL:EXT:cms/locallang_ttc.php:header',
 			'config' => Array (
 				'type' => 'input',
@@ -196,6 +204,7 @@ $TCA['tt_content'] = Array (
 		),
 		'header_position' => Array (
 			'label' => 'LLL:EXT:cms/locallang_ttc.php:header_position',
+			'exclude' => 1,
 			'config' => Array (
 				'type' => 'select',
 				'items' => Array (
@@ -209,6 +218,7 @@ $TCA['tt_content'] = Array (
 		),
 		'header_link' => Array (
 			'label' => 'LLL:EXT:cms/locallang_ttc.php:header_link',
+			'exclude' => 1,			
 			'config' => Array (
 				'type' => 'input',
 				'size' => '15',
@@ -224,7 +234,8 @@ $TCA['tt_content'] = Array (
 						'script' => 'browse_links.php?mode=wizard',
 						'JSopenParams' => 'height=300,width=500,status=0,menubar=0,scrollbars=1'
 					)
-				)
+				),
+				'softref' => 'typolink'
 			)
 		),
 		'header_layout' => Array (
@@ -250,11 +261,13 @@ $TCA['tt_content'] = Array (
 			'config' => Array (
 				'type' => 'input',
 				'size' => '30',
-				'max' => '256'
+				'max' => '256',
+				'softref' => 'email[subst]'
 			)
 		),
 		'bodytext' => Array (
 			'l10n_mode' => 'prefixLangTitle',
+			'l10n_cat' => 'text',
 			'label' => 'LLL:EXT:lang/locallang_general.php:LGL.text',
 			'config' => Array (
 				'type' => 'text',
@@ -289,7 +302,8 @@ $TCA['tt_content'] = Array (
 						'script' => 'wizard_forms.php?special=formtype_mail',
 						'params' => array('xmlOutput' => 0)
 					)
-				)
+				),
+				'softref' => 'typolink_tag,images,email[subst],url'
 			)
 		),
 		'text_align' => Array (
@@ -385,7 +399,7 @@ $TCA['tt_content'] = Array (
 				'type' => 'group',
 				'internal_type' => 'file',
 				'allowed' => $GLOBALS['TYPO3_CONF_VARS']['GFX']['imagefile_ext'],
-				'max_size' => '1000',
+				'max_size' => $GLOBALS['TYPO3_CONF_VARS']['BE']['maxFileSize'],
 				'uploadfolder' => 'uploads/pics',
 				'show_thumbs' => '1',
 				'size' => '3',
@@ -444,7 +458,7 @@ $TCA['tt_content'] = Array (
 					Array('LLL:EXT:cms/locallang_ttc.php:imageorient.I.10', 26, 'selicons/intext_left_nowrap.gif')
 				),
 				'selicon_cols' => 6,
-				'default' => '8',
+				'default' => '0',
 				'iconsInOptionTags' => 1,
 			)
 		),
@@ -480,7 +494,8 @@ $TCA['tt_content'] = Array (
 						'script' => 'browse_links.php?mode=wizard',
 						'JSopenParams' => 'height=300,width=500,status=0,menubar=0,scrollbars=1'
 					)
-				)
+				),
+				'softref' => 'typolink[linkList]'
 			)
 		),
 		'image_zoom' => Array (
@@ -578,7 +593,8 @@ $TCA['tt_content'] = Array (
 			'config' => Array (
 				'type' => 'text',
 				'cols' => '30',
-				'rows' => '3'
+				'rows' => '3',
+				'softref' => 'typolink_tag,images,email[subst],url'
 			)
 		),
 		'imagecaption_position' => Array (
@@ -595,6 +611,33 @@ $TCA['tt_content'] = Array (
 				'default' => ''
 			)
 		),
+		'altText' => Array (
+			'exclude' => 1,
+			'label' => 'LLL:EXT:cms/locallang_ttc.php:image_altText',
+			'config' => Array (
+				'type' => 'text',
+				'cols' => '30',
+				'rows' => '3'
+			)
+		),
+		'titleText' => Array (
+			'exclude' => 1,
+			'label' => 'LLL:EXT:cms/locallang_ttc.php:image_titleText',
+			'config' => Array (
+				'type' => 'text',
+				'cols' => '30',
+				'rows' => '3'
+			)
+		),
+		'longdescURL' => Array (
+			'exclude' => 1,
+			'label' => 'LLL:EXT:cms/locallang_ttc.php:image_longdescURL',
+			'config' => Array (
+				'type' => 'text',
+				'cols' => '30',
+				'rows' => '3'
+			)
+		),
 		'cols' => Array (
 			'label' => 'LLL:EXT:cms/locallang_ttc.php:cols',
 			'config' => Array (
@@ -754,8 +797,8 @@ $TCA['tt_content'] = Array (
 				'type' => 'group',
 				'internal_type' => 'file',
 				'allowed' => '',	// Must be empty for disallowed to work.
-				'disallowed' => 'php,php3',
-				'max_size' => '10000',
+				'disallowed' => PHP_EXTENSIONS_DEFAULT,
+				'max_size' => $GLOBALS['TYPO3_CONF_VARS']['BE']['maxFileSize'],
 				'uploadfolder' => 'uploads/media',
 				'show_thumbs' => '1',
 				'size' => '3',
@@ -769,7 +812,7 @@ $TCA['tt_content'] = Array (
 				'type' => 'group',
 				'internal_type' => 'file',
 				'allowed' => 'txt,html,htm,class,swf,swa,dcr,wav,avi,au,mov,asf,mpg,wmv,mp3',
-				'max_size' => '10000',
+				'max_size' => $GLOBALS['TYPO3_CONF_VARS']['BE']['maxFileSize'],
 				'uploadfolder' => 'uploads/media',
 				'size' => '2',
 				'maxitems' => '1',
@@ -885,10 +928,11 @@ $TCA['tt_content'] = Array (
 			)
 		),
 		'pi_flexform' => array(
+			'l10n_display' => 'hideDiff',
 			'label' => 'LLL:EXT:cms/locallang_ttc.php:pi_flexform',
 			'config' => Array (
 				'type' => 'flex',
-				'ds_pointerField' => 'list_type',
+				'ds_pointerField' => 'list_type,CType',
 				'ds' => array(
 					'default' => '
 						<T3DataStructure>
@@ -925,39 +969,104 @@ $TCA['tt_content'] = Array (
 	),
 	'types' => Array (
 		'1' => 	Array('showitem' => 'CType'),
-		'header' => 	Array('showitem' => 'CType;;4;button;1-1-1, header;;3;;2-2-2, subheader;;8'),
-		'text' => 		Array('showitem' => 'CType;;4;button;1-1-1, header;;3;;2-2-2, bodytext;;9;richtext[paste|bold|italic|underline|formatblock|class|left|center|right|orderedlist|unorderedlist|outdent|indent|link|image]:rte_transform[flag=rte_enabled|mode=ts];3-3-3, rte_enabled, text_properties'),
-		'textpic' => 	Array('showitem' => 'CType;;4;button;1-1-1, header;;3;;2-2-2, bodytext;;9;richtext[paste|bold|italic|underline|formatblock|class|left|center|right|orderedlist|unorderedlist|outdent|indent|link|image]:rte_transform[flag=rte_enabled|mode=ts];3-3-3, rte_enabled, text_properties, --div--, image;;;;4-4-4, imageorient;;2, imagewidth;;13,
-			--palette--;LLL:EXT:cms/locallang_ttc.php:ALT.imgLinks;7,
-			--palette--;LLL:EXT:cms/locallang_ttc.php:ALT.imgOptions;11,
-			imagecaption;;5'),
-		'rte' => 		Array('showitem' => 'CType;;4;button;1-1-1, header;;3;;2-2-2, bodytext;;;nowrap:richtext[*]:rte_transform[mode=ts_images-ts_reglinks];3-3-3'),
-		'image' => 		Array('showitem' => 'CType;;4;button;1-1-1, header;;3;;2-2-2, image;;;;4-4-4, imageorient;;2, imagewidth;;13,
-			--palette--;LLL:EXT:cms/locallang_ttc.php:ALT.imgLinks;7,
-			--palette--;LLL:EXT:cms/locallang_ttc.php:ALT.imgOptions;11,
-			 imagecaption;;5'),
-		'bullets' => 	Array('showitem' => 'CType;;4;button;1-1-1, header;;3;;2-2-2, layout;;;;3-3-3, bodytext;;9;nowrap, text_properties'),
-		'table' => 		Array('showitem' => 'CType;;4;button;1-1-1, header;;3;;2-2-2, layout;;10;button;3-3-3, cols, bodytext;;9;nowrap:wizards[table], text_properties'),
-		'splash' => 	Array('showitem' => 'CType;;4;button;1-1-1, header;LLL:EXT:lang/locallang_general.php:LGL.name;;;2-2-2, splash_layout, bodytext;;;;3-3-3, image;;6'),
-		'uploads' => 	Array('showitem' => 'CType;;4;button;1-1-1, header;;3;;2-2-2, media;;;;5-5-5,
-			select_key;LLL:EXT:cms/locallang_ttc.php:select_key.ALT.uploads,
-			layout;;10;button, filelink_size,
-			imagecaption;LLL:EXT:cms/locallang_ttc.php:imagecaption.ALT.uploads;;nowrap'),
-		'multimedia' =>	Array('showitem' => 'CType;;4;button;1-1-1, header;;3;;2-2-2, multimedia;;;;5-5-5, bodytext;LLL:EXT:lang/locallang_general.php:LGL.parameters;;nowrap'),
-		'script' =>		Array('showitem' => 'CType;;4;button;1-1-1, header;LLL:EXT:lang/locallang_general.php:LGL.name;;;2-2-2, select_key;;;;5-5-5, pages;;12, bodytext;LLL:EXT:lang/locallang_general.php:LGL.parameters;;nowrap,
-			imagecaption;LLL:EXT:cms/locallang_ttc.php:imagecaption.ALT.script'),
-		'menu' => 		Array('showitem' => 'CType;;4;button;1-1-1, header;;3;;2-2-2, menu_type;;;;5-5-5, pages'),
-		'mailform' => 	Array('showitem' => 'CType;;4;button;1-1-1, header;;3;;2-2-2,
-			bodytext;LLL:EXT:cms/locallang_ttc.php:bodytext.ALT.mailform;;nowrap:wizards[forms];5-5-5,
-			pages;LLL:EXT:cms/locallang_ttc.php:pages.ALT.mailform,
-			subheader;LLL:EXT:cms/locallang_ttc.php:subheader.ALT.mailform'),
-		'search' => 	Array('showitem' => 'CType;;4;button;1-1-1, header;;3;;2-2-2,
-			pages;LLL:EXT:cms/locallang_ttc.php:pages.ALT.search;;;5-5-5'),
-		'login' => 		Array('showitem' => 'CType;;4;button;1-1-1, header;;3;;2-2-2,
-			pages;LLL:EXT:cms/locallang_ttc.php:pages.ALT.login;;;5-5-5'),
-		'shortcut' => 	Array('showitem' => 'CType;;4;button;1-1-1, header;LLL:EXT:lang/locallang_general.php:LGL.name;;;2-2-2, records;;;;5-5-5, layout'),
+		'header' => 	Array(
+			'showitem' => 'CType;;4;button,hidden,1-1-1, header;;3;;2-2-2, subheader;;8, linkToTop;;;;3-3-3,
+			--div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.access, starttime, endtime'
+		),
+		'text' => 		Array(
+			'showitem' => 'CType;;4;button,hidden,1-1-1, header;;3;;2-2-2,linkToTop;;;;3-3-3,
+							--div--;LLL:EXT:cms/locallang_ttc.xml:CType.I.1, bodytext;;9;richtext:rte_transform[flag=rte_enabled|mode=ts_css];3-3-3, rte_enabled, text_properties,
+							--div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.access, starttime, endtime',
+		),
+		'textpic' => 	Array(
+			'showitem' => 'CType;;4;button,hidden,1-1-1, header;;3;;2-2-2, linkToTop;;;;3-3-3,
+							--div--;LLL:EXT:cms/locallang_ttc.xml:tabs.text,bodytext;;9;richtext:rte_transform[flag=rte_enabled|mode=ts_css];3-3-3, rte_enabled, text_properties,
+							--div--;LLL:EXT:cms/locallang_ttc.xml:tabs.media, image;;;;5-5-5, imageorient;;2,
+							--palette--;LLL:EXT:cms/locallang_ttc.php:ALT.imgDimensions;13,;;;;6-6-6,
+							--palette--;LLL:EXT:cms/locallang_ttc.php:ALT.imgLinks;7,
+							imagecaption;;5,altText;;;;7-7-7,titleText,longdescURL,;;;;8-8-8,
+							--palette--;LLL:EXT:cms/locallang_ttc.php:ALT.imgOptions;11,
+							--div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.access, starttime, endtime'
+		),
+		'rte' => 		Array(
+			'showitem' => 'CType;;4;button,hidden,1-1-1, header;;3;;2-2-2,
+							--div--;LLL:EXT:cms/locallang_ttc.xml:tabs.text, bodytext;;;nowrap:richtext[*]:rte_transform[mode=ts_images-ts_reglinks];3-3-3,
+							--div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.access, starttime, endtime'
+		),
+		'image' => 		Array(
+			'showitem' => 'CType;;4;button,hidden,1-1-1, header;;3;;2-2-2, linkToTop;;;;3-3-3,
+							--div--;LLL:EXT:cms/locallang_ttc.xml:tabs.media, image;;;;4-4-4, imageorient;;2,
+							--palette--;LLL:EXT:cms/locallang_ttc.php:ALT.imgDimensions;13,;;;;5-5-5,
+							--palette--;LLL:EXT:cms/locallang_ttc.php:ALT.imgLinks;7,
+							imagecaption;;5,altText;;;;6-6-6,titleText,longdescURL,;;;;7-7-7,
+							--palette--;LLL:EXT:cms/locallang_ttc.php:ALT.imgOptions;11,
+			 				--div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.access,starttime, endtime'
+		),
+		'bullets' => 	Array(
+			'showitem' => 'CType;;4;button,hidden,1-1-1, header;;3;;2-2-2, linkToTop;;;;4-4-4,
+							--div--;LLL:EXT:cms/locallang_ttc.xml:CType.I.4, layout;;;;3-3-3, bodytext;;9;nowrap, text_properties,
+							--div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.access,starttime, endtime'
+		),
+		'table' => 		Array(
+			'showitem' => 'CType;;4;button,hidden,1-1-1, header;;3;;2-2-2, linkToTop;;;;4-4-4,
+							--div--;LLL:EXT:cms/locallang_ttc.xml:CType.I.5, layout;;10;button;3-3-3, cols, bodytext;;9;nowrap:wizards[table], text_properties,
+							--div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.access,starttime, endtime'
+		),
+		'splash' => 	Array(
+			'showitem' => 'CType;;4;button,hidden,1-1-1, header;LLL:EXT:lang/locallang_general.php:LGL.name;;;2-2-2,
+							--div--;LLL:EXT:cms/locallang_ttc.xml:CType.I.11, splash_layout, bodytext;;;;3-3-3, image;;6,
+							--div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.access,starttime, endtime'
+		),
+		'uploads' => 	Array(
+			'showitem' => 'CType;;4;button,hidden,1-1-1, header;;3;;2-2-2, linkToTop;;;;3-3-3,
+							--div--;LLL:EXT:cms/locallang_ttc.xml:CType.I.6, media;;;;3-3-3,
+							select_key;LLL:EXT:cms/locallang_ttc.php:select_key.ALT.uploads,
+							layout;;10;button, filelink_size,
+							imagecaption;LLL:EXT:cms/locallang_ttc.php:imagecaption.ALT.uploads;;nowrap,;;;;4-4-4,
+							--div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.access,starttime, endtime'
+		),
+		'multimedia' =>	Array(
+			'showitem' => 'CType;;4;button,hidden,1-1-1, header;;3;;2-2-2, linkToTop;;;;3-3-3,
+							--div--;LLL:EXT:cms/locallang_ttc.xml:CType.I.7, multimedia;;;;3-3-3, bodytext;LLL:EXT:lang/locallang_general.php:LGL.parameters;;nowrap,
+							--div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.access,starttime, endtime'
+		),
+		'script' =>		Array(
+			'showitem' => 'CType;;4;button,hidden,1-1-1, header;LLL:EXT:lang/locallang_general.php:LGL.name;;;2-2-2,
+							--div--;LLL:EXT:cms/locallang_ttc.xml:CType.I.15, select_key;;;;3-3-3, pages;;12, bodytext;LLL:EXT:lang/locallang_general.php:LGL.parameters;;nowrap,
+							imagecaption;LLL:EXT:cms/locallang_ttc.php:imagecaption.ALT.script,;;;;4-4-4,linkToTop,
+							--div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.access,starttime, endtime'
+		),
+		'menu' => 		Array(
+			'showitem' => 'CType;;4;button,hidden,1-1-1, header;;3;;2-2-2,  linkToTop;;;;3-3-3,
+							--div--;LLL:EXT:cms/locallang_ttc.xml:CType.I.12, menu_type;;;;4-4-4, pages,
+							--div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.access,starttime, endtime'
+		),
+		'mailform' => 	Array(
+			'showitem' => 'CType;;4;button,hidden,1-1-1, header;;3;;2-2-2, linkToTop;;;;3-3-3,
+							--div--;LLL:EXT:cms/locallang_ttc.xml:CType.I.8, bodytext;LLL:EXT:cms/locallang_ttc.php:bodytext.ALT.mailform;;nowrap:wizards[forms];3-3-3,
+							pages;LLL:EXT:cms/locallang_ttc.php:pages.ALT.mailform,
+							subheader;LLL:EXT:cms/locallang_ttc.php:subheader.ALT.mailform,
+							--div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.access,starttime, endtime'
+		),
+		'search' => 	Array(
+			'showitem' => 'CType;;4;button,hidden,1-1-1, header;;3;;2-2-2, linkToTop;;;;3-3-3,
+							--div--;LLL:EXT:cms/locallang_ttc.xml:CType.I.9, pages;LLL:EXT:cms/locallang_ttc.php:pages.ALT.search,
+							--div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.access,starttime, endtime'
+		),
+		'login' => 		Array(
+			'showitem' => 'CType;;4;button,hidden,1-1-1, header;;3;;2-2-2, linkToTop;;;;3-3-3,
+							--div--;LLL:EXT:cms/locallang_ttc.xml:CType.I.10, pages;LLL:EXT:cms/locallang_ttc.php:pages.ALT.login,
+							--div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.access,starttime, endtime'
+		),
+		'shortcut' => 	Array(
+			'showitem' => 'CType;;4;button,hidden,1-1-1, header;LLL:EXT:lang/locallang_general.php:LGL.name;;;2-2-2,
+							--div--;LLL:EXT:cms/locallang_ttc.xml:CType.I.13, records;;;;5-5-5, layout,
+							--div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.access,starttime, endtime'
+		),
 		'list' => 		Array(
-							'showitem' => 'CType;;4;button;1-1-1, header;;3;;2-2-2, --div--, list_type;;;;5-5-5, layout, select_key, pages;;12',
+							'showitem' => 'CType;;4;button,hidden,1-1-1, header;;3;;2-2-2, linkToTop;;;;3-3-3,
+							--div--;LLL:EXT:cms/locallang_ttc.xml:tabs.plugin, list_type;;;;3-3-3, layout, select_key, pages;;12,
+							--div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.access,starttime, endtime',
 							'subtype_value_field' => 'list_type',
 							'subtypes_excludelist' => Array(
 								'' => 'layout,select_key,pages',	// When no plugin is selected.
@@ -971,34 +1080,39 @@ $TCA['tt_content'] = Array (
 								'7' => 'layout',
 								'1' => 'layout',
 								'8' => 'layout',
-								'indexed_search' => 'layout',
 								'11' => 'layout',
 								'20' => 'layout',
 								'21' => 'layout'
 							)
 						),
-		'div' => 		Array('showitem' => 'CType;;14;button;1-1-1, header;LLL:EXT:lang/locallang_general.php:LGL.name;;;2-2-2'),
-		'html' => 		Array('showitem' => 'CType;;4;button;1-1-1, header;LLL:EXT:lang/locallang_general.php:LGL.name;;;2-2-2,
-			bodytext;LLL:EXT:cms/locallang_ttc.php:bodytext.ALT.html;;nowrap;3-3-3')
+		'div' => 		Array(
+			'showitem' => 'CType;;14;button,hidden,1-1-1, header;LLL:EXT:lang/locallang_general.php:LGL.name;;;2-2-2,linkToTop,
+							--div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.access,starttime, endtime'
+		),
+		'html' => 		Array(
+			'showitem' => 'CType;;4;button,hidden,1-1-1, header;LLL:EXT:lang/locallang_general.php:LGL.name;;;2-2-2, linkToTop;;;;3-3-3,
+							--div--;LLL:EXT:cms/locallang_ttc.xml:CType.I.17, bodytext;LLL:EXT:cms/locallang_ttc.php:bodytext.ALT.html;;nowrap,
+							--div--;LLL:EXT:cms/locallang_tca.xml:pages.tabs.access,starttime, endtime'
+		)
 	),
 	'palettes' => Array (
-		'1' => Array('showitem' => 'hidden, starttime, endtime, fe_group'),
+		'1' => Array('showitem' => 'starttime, endtime'),
 		'2' => Array('showitem' => 'imagecols, image_noRows, imageborder'),
 		'3' => Array('showitem' => 'header_position, header_layout, header_link, date'),
-		'4' => Array('showitem' => 'sys_language_uid, l18n_parent, colPos, spaceBefore, spaceAfter, section_frame, sectionIndex, linkToTop'),
+		'4' => Array('showitem' => 'sys_language_uid, l18n_parent, colPos, spaceBefore, spaceAfter, section_frame, sectionIndex'),
 		'5' => Array('showitem' => 'imagecaption_position'),
 		'6' => Array('showitem' => 'imagewidth,image_link'),
-		'7' => Array('showitem' => 'image_link, image_zoom'),
+		'7' => Array('showitem' => 'image_link, image_zoom','canNotCollapse' => 1),
 		'8' => Array('showitem' => 'layout'),
 		'9' => Array('showitem' => 'text_align,text_face,text_size,text_color'),
 		'10' => Array('showitem' => 'table_bgColor, table_border, table_cellspacing, table_cellpadding'),
-		'11' => Array('showitem' => 'image_compression, image_effects, image_frames'),
+		'11' => Array('showitem' => 'image_compression, image_effects, image_frames','canNotCollapse' => 1),
 		'12' => Array('showitem' => 'recursive'),
-		'13' => Array('showitem' => 'imageheight'),
-		'14' => Array('showitem' => 'sys_language_uid, l18n_parent, colPos')
+		'13' => Array('showitem' => 'imagewidth, imageheight','canNotCollapse' => 1),
+		'14' => Array('showitem' => 'sys_language_uid, l18n_parent, colPos'),
+		'15' => Array('showitem' => 'fe_group'),
 	)
 );
 
 
-
-?>
+?>
\ No newline at end of file
