@@ -2,7 +2,7 @@
 /***************************************************************
 *  Copyright notice
 *
-*  (c) 1999-2005 Kasper Skaarhoj (kasperYYYY@typo3.com)
+*  (c) 1999-2008 Kasper Skaarhoj (kasperYYYY@typo3.com)
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
@@ -28,7 +28,7 @@
  * Libraries for pagegen.php
  * The script "pagegen.php" is included by "index_ts.php" when a page is not cached but needs to be rendered.
  *
- * $Id: class.tslib_pagegen.php 1657 2006-07-26 17:54:15Z mundaun $
+ * $Id: class.tslib_pagegen.php 3759 2008-06-05 10:01:32Z ohader $
  * Revised for TYPO3 3.6 June/2003 by Kasper Skaarhoj
  * XHTML compliant
  *
@@ -150,42 +150,6 @@ See <a href="http://wiki.typo3.org/index.php/TYPO3_3.8.1" target="_blank">wiki.t
 			$GLOBALS['TSFE']->spamProtectEmailAddresses = 'ascii';
 		} else {
 			$GLOBALS['TSFE']->spamProtectEmailAddresses = t3lib_div::intInRange($GLOBALS['TSFE']->config['config']['spamProtectEmailAddresses'],-10,10,0);
-			if ($GLOBALS['TSFE']->spamProtectEmailAddresses)	{
-				$GLOBALS['TSFE']->additionalJavaScript['UnCryptMailto()']='
-  // decrypt helper function
-function decryptCharcode(n,start,end,offset) {
-	n = n + offset;
-	if (offset > 0 && n > end)	{
-		n = start + (n - end - 1);
-	} else if (offset < 0 && n < start)	{
-		n = end - (start - n - 1);
-	}
-	return String.fromCharCode(n);
-}
-  // decrypt string
-function decryptString(enc,offset) {
-	var dec = "";
-	var len = enc.length;
-	for(var i=0; i < len; i++)	{
-		var n = enc.charCodeAt(i);
-		if (n >= 0x2B && n <= 0x39)	{
-			dec += decryptCharcode(n,0x2B,0x3A,offset);	// 0-9 . , - + / :
-		} else if (n >= 0x40 && n <= 0x5A)	{
-			dec += decryptCharcode(n,0x40,0x5A,offset);	// A-Z @
-		} else if (n >= 0x61 && n <= 0x7A)	{
-			dec += decryptCharcode(n,0x61,0x7A,offset);	// a-z
-		} else {
-			dec += enc.charAt(i);
-		}
-	}
-	return dec;
-}
-  // decrypt spam-protected emails
-function linkTo_UnCryptMailto(s)	{
-	location.href = decryptString(s,'.($GLOBALS['TSFE']->spamProtectEmailAddresses*-1).');
-}
-'				;
-			}
 		}
 
 
@@ -230,22 +194,41 @@ function linkTo_UnCryptMailto(s)	{
 		}
 
 			// linkVars
-		$GLOBALS['TSFE']->linkVars = ''.$GLOBALS['TSFE']->config['config']['linkVars'];
-		if ($GLOBALS['TSFE']->linkVars)	{
-			$linkVarArr = explode(',',$GLOBALS['TSFE']->linkVars);
+		$linkVars = (string)$GLOBALS['TSFE']->config['config']['linkVars'];
+		if ($linkVars)	{
+			$linkVarArr = explode(',',$linkVars);
+
 			$GLOBALS['TSFE']->linkVars='';
-			reset($linkVarArr);
-			while(list(,$val)=each($linkVarArr))	{
-				$val=trim($val);
-				$GET = t3lib_div::_GET();
+			$GET = t3lib_div::_GET();
+
+			foreach ($linkVarArr as $val)	{
+				$val = trim($val);
+
+				if (preg_match('/^(.*)\((.+)\)$/',$val,$match))	{
+					$val = trim($match[1]);
+					$test = trim($match[2]);
+				} else unset($test);
+
 				if ($val && isset($GET[$val]))	{
 					if (!is_array($GET[$val]))	{
-						$GLOBALS['TSFE']->linkVars.='&'.$val.'='.rawurlencode($GET[$val]);
+						$tmpVal = rawurlencode($GET[$val]);
+
+						if ($test && !TSpagegen::isAllowedLinkVarValue($tmpVal,$test))	{
+							continue;	// Error: This value was not allowed for this key
+						}
+
+						$value = '&'.$val.'='.$tmpVal;
 					} else {
-						$GLOBALS['TSFE']->linkVars.=t3lib_div::implodeArrayForUrl($val,$GET[$val]);
+						if ($test && strcmp('array',$test))	{
+							continue;	// Error: This key must not be an array!
+						}
+						$value = t3lib_div::implodeArrayForUrl($val,$GET[$val]);
 					}
-				}
+				} else continue;
+
+				$GLOBALS['TSFE']->linkVars.= $value;
 			}
+			unset($GET);
 		} else {
 			$GLOBALS['TSFE']->linkVars='';
 		}
@@ -320,32 +303,21 @@ function linkTo_UnCryptMailto(s)	{
 	 * @return	array		Array with a) a JavaScript section with event handlers and variables set and b) an array with attributes for the body tag.
 	 */
 	function JSeventFunctions()	{
-		$functions=array();
-		$setEvents=array();
-		$setBody=array();
+		$functions = array();
+		$setEvents = array();
+		$setBody = array();
 
-		if (is_array($GLOBALS['TSFE']->JSeventFuncCalls['onmousemove']) && count($GLOBALS['TSFE']->JSeventFuncCalls['onmousemove']))	{
-			$functions[]='	function T3_onmousemoveWrapper(e)	{	'.implode('   ',$GLOBALS['TSFE']->JSeventFuncCalls['onmousemove']).'	}';
-			$setEvents[]='	document.onmousemove=T3_onmousemoveWrapper;';
-		}
-		if (is_array($GLOBALS['TSFE']->JSeventFuncCalls['onmouseup']) && count($GLOBALS['TSFE']->JSeventFuncCalls['onmouseup']))	{
-			$functions[]='	function T3_onmouseupWrapper(e)	{	'.implode('   ',$GLOBALS['TSFE']->JSeventFuncCalls['onmouseup']).'	}';
-			$setEvents[]='	document.onmouseup=T3_onmouseupWrapper;';
-		}
-		if (is_array($GLOBALS['TSFE']->JSeventFuncCalls['onload']) && count($GLOBALS['TSFE']->JSeventFuncCalls['onload']))	{
-			$functions[]='	function T3_onloadWrapper()	{	'.implode('   ',$GLOBALS['TSFE']->JSeventFuncCalls['onload']).'	}';
-			$setEvents[]='	document.onload=T3_onloadWrapper;';
-			$setBody[]='onload="T3_onloadWrapper();"';
+		foreach ($GLOBALS['TSFE']->JSeventFuncCalls as $event => $handlers)	{
+			if (count($handlers))	{
+				$functions[] = '	function T3_'.$event.'Wrapper(e)	{	'.implode('   ',$handlers).'	}';
+				$setEvents[] = '	document.'.$event.'=T3_'.$event.'Wrapper;';
+				if ($event == 'onload')	{
+					$setBody[]='onload="T3_onloadWrapper();"';	// dubiuos double setting breaks on some browser - do we need it?
+				}
+			}
 		}
 
-		return Array(count($functions)?'
-<script type="text/javascript">
-	/*<![CDATA[*/
-'.implode(chr(10),$functions).'
-'.implode(chr(10),$setEvents).'
-	/*]]>*/
-</script>
-			':'',$setBody);
+		return array(count($functions)? implode(chr(10), $functions) . chr(10) . implode(chr(10), $setEvents) : '', $setBody);
 	}
 
 	/**
@@ -354,7 +326,7 @@ function linkTo_UnCryptMailto(s)	{
 	 * @return	void
 	 */
 	function renderContent()	{
-		// PAGE CONTENT
+			// PAGE CONTENT
 		$GLOBALS['TT']->incStackPointer();
 		$GLOBALS['TT']->push($GLOBALS['TSFE']->sPre, 'PAGE');
 			$pageContent = $GLOBALS['TSFE']->cObj->cObjGet($GLOBALS['TSFE']->pSetup);
@@ -494,12 +466,12 @@ function linkTo_UnCryptMailto(s)	{
 		$GLOBALS['TSFE']->content.='
 	<meta http-equiv="Content-Type" content="text/html; charset='.$theCharset.'" />';
 
-$GLOBALS['TSFE']->content.='
+		$GLOBALS['TSFE']->content.='
 
 <!-- '.($customContent?$customContent.chr(10):'').'
 	This website is powered by TYPO3 - inspiring people to share!
 	TYPO3 is a free open source Content Management Framework initially created by Kasper Skaarhoj and licensed under GNU/GPL.
-	TYPO3 is copyright 1998-2006 of Kasper Skaarhoj. Extensions are copyright of their respective owners.
+	TYPO3 is copyright 1998-2008 of Kasper Skaarhoj. Extensions are copyright of their respective owners.
 	Information and contribution at http://typo3.com/ and http://typo3.org/
 -->
 ';
@@ -511,9 +483,17 @@ $GLOBALS['TSFE']->content.='
 		}
 
 		if ($GLOBALS['TSFE']->pSetup['shortcutIcon']) {
-			$ss=$path.$GLOBALS['TSFE']->tmpl->getFileName($GLOBALS['TSFE']->pSetup['shortcutIcon']);
-			$GLOBALS['TSFE']->content.='
-	<link rel="SHORTCUT ICON" href="'.htmlspecialchars($ss).'" />';
+			$favIcon = t3lib_div::getIndpEnv('TYPO3_SITE_URL') . $GLOBALS['TSFE']->tmpl->getFileName($GLOBALS['TSFE']->pSetup['shortcutIcon']);
+			$iconMimeType = '';
+			if (function_exists('finfo_open')) {
+				$finfo = finfo_open(FILEINFO_MIME);
+				$iconMimeType = ' type="'.finfo_file($finfo,$favIcon).'"';
+				finfo_close($finfo);
+			}
+
+			$GLOBALS['TSFE']->content.= '
+	<link rel="shortcut icon" href="'.htmlspecialchars($favIcon).'"'.$iconMimeType.' />
+	<link rel="icon" href="'.htmlspecialchars($favIcon).'"'.$iconMimeType.' />';
 		}
 
 			// Including CSS files
@@ -541,7 +521,7 @@ $GLOBALS['TSFE']->content.='
 		}
 
 		if ($GLOBALS['TSFE']->pSetup['stylesheet'])	{
-			$ss=$GLOBALS['TSFE']->tmpl->getFileName($GLOBALS['TSFE']->pSetup['stylesheet']);
+			$ss = $GLOBALS['TSFE']->tmpl->getFileName($GLOBALS['TSFE']->pSetup['stylesheet']);
 			if ($ss)	{
 				$GLOBALS['TSFE']->content.='
 	<link rel="stylesheet" type="text/css" href="'.htmlspecialchars($ss).'" />';
@@ -727,13 +707,13 @@ $GLOBALS['TSFE']->content.='
 				// Storing the JSCode and JSImgCode vars...
 			$GLOBALS['TSFE']->additionalHeaderData['JSCode'] = $GLOBALS['TSFE']->JSCode;
 			$GLOBALS['TSFE']->additionalHeaderData['JSImgCode'] = $GLOBALS['TSFE']->JSImgCode;
-			$GLOBALS['TSFE']->config['INTincScript_ext']['divKey']= $GLOBALS['TSFE']->uniqueHash();
-			$GLOBALS['TSFE']->config['INTincScript_ext']['additionalHeaderData']	= $GLOBALS['TSFE']->additionalHeaderData;	// Storing the header-data array
-			$GLOBALS['TSFE']->config['INTincScript_ext']['additionalJavaScript']	= $GLOBALS['TSFE']->additionalJavaScript;	// Storing the JS-data array
-			$GLOBALS['TSFE']->config['INTincScript_ext']['additionalCSS']	= $GLOBALS['TSFE']->additionalCSS;	// Storing the Style-data array
+			$GLOBALS['TSFE']->config['INTincScript_ext']['divKey'] = $GLOBALS['TSFE']->uniqueHash();
+			$GLOBALS['TSFE']->config['INTincScript_ext']['additionalHeaderData'] = $GLOBALS['TSFE']->additionalHeaderData;	// Storing the header-data array
+			$GLOBALS['TSFE']->config['INTincScript_ext']['additionalJavaScript'] = $GLOBALS['TSFE']->additionalJavaScript;	// Storing the JS-data array
+			$GLOBALS['TSFE']->config['INTincScript_ext']['additionalCSS'] = $GLOBALS['TSFE']->additionalCSS;	// Storing the Style-data array
 
-			$GLOBALS['TSFE']->additionalHeaderData=array('<!--HD_'.$GLOBALS['TSFE']->config['INTincScript_ext']['divKey'].'-->');	// Clearing the array
-			$GLOBALS['TSFE']->divSection.='<!--TDS_'.$GLOBALS['TSFE']->config['INTincScript_ext']['divKey'].'-->';
+			$GLOBALS['TSFE']->additionalHeaderData = array('<!--HD_'.$GLOBALS['TSFE']->config['INTincScript_ext']['divKey'].'-->');	// Clearing the array
+			$GLOBALS['TSFE']->divSection.= '<!--TDS_'.$GLOBALS['TSFE']->config['INTincScript_ext']['divKey'].'-->';
 		} else {
 			$GLOBALS['TSFE']->INTincScript_loadJSCode();
 		}
@@ -750,26 +730,103 @@ $GLOBALS['TSFE']->content.='
 		function blurLink(theObject)	{	//
 			if (msie4)	{theObject.blur();}
 		}
+		' . $JSef[0];
+
+		if ($GLOBALS['TSFE']->spamProtectEmailAddresses && $GLOBALS['TSFE']->spamProtectEmailAddresses !== 'ascii') {
+			$_scriptCode.= '
+			// decrypt helper function
+		function decryptCharcode(n,start,end,offset)	{
+			n = n + offset;
+			if (offset > 0 && n > end)	{
+				n = start + (n - end - 1);
+			} else if (offset < 0 && n < start)	{
+				n = end - (start - n - 1);
+			}
+			return String.fromCharCode(n);
+		}
+			// decrypt string
+		function decryptString(enc,offset)	{
+			var dec = "";
+			var len = enc.length;
+			for(var i=0; i < len; i++)	{
+				var n = enc.charCodeAt(i);
+				if (n >= 0x2B && n <= 0x3A)	{
+					dec += decryptCharcode(n,0x2B,0x3A,offset);	// 0-9 . , - + / :
+				} else if (n >= 0x40 && n <= 0x5A)	{
+					dec += decryptCharcode(n,0x40,0x5A,offset);	// A-Z @
+				} else if (n >= 0x61 && n <= 0x7A)	{
+					dec += decryptCharcode(n,0x61,0x7A,offset);	// a-z
+				} else {
+					dec += enc.charAt(i);
+				}
+			}
+			return dec;
+		}
+			// decrypt spam-protected emails
+		function linkTo_UnCryptMailto(s)	{
+			location.href = decryptString(s,'.($GLOBALS['TSFE']->spamProtectEmailAddresses*-1).');
+		}
 		';
+		}
+
+			//add inline JS
+		$_inlineJS = '';
+			// defined in TS with page.inlineJS
+		if (is_array($GLOBALS['TSFE']->pSetup['inlineJS.']))	{
+			$GLOBALS['TSFE']->inlineJS[]= $GLOBALS['TSFE']->cObj->cObjGet($GLOBALS['TSFE']->pSetup['inlineJS.'],'inlineJS.');
+		}
+			// defined in php
+		if(is_array($GLOBALS['TSFE']->inlineJS)) {
+			foreach($GLOBALS['TSFE']->inlineJS as $key=>$val) {
+				if(!is_array($val)) {
+					$_inlineJS .= chr(10).$val.chr(10);
+				}
+			}
+		}
+
+			// Should minify?
+		if ($GLOBALS['TSFE']->config['config']['minifyJS']) {
+			$minifyErrorScript = $minifyErrorInline = '';
+			$_scriptCode = t3lib_div::minifyJavaScript($_scriptCode,$minifyErrorScript);
+			if ($minifyErrorScript) {
+				$GLOBALS['TT']->setTSlogMessage($minifyErrorScript, 3);
+			}
+			if ($_inlineJS) {
+				$_inlineJS = t3lib_div::minifyJavaScript($_inlineJS,$minifyErrorInline);
+				if ($minifyErrorInline) {
+					$GLOBALS['TT']->setTSlogMessage($minifyErrorInline, 3);
+				}
+			}
+		}
+
 		if (!$GLOBALS['TSFE']->config['config']['removeDefaultJS']) {
-				// NOTICE: The following code must be kept synchronized with "tslib/default.js"!!!
+				// inlude default and inlineJS
 			$GLOBALS['TSFE']->content.='
 	<script type="text/javascript">
 		/*<![CDATA[*/
-	<!--'.$_scriptCode.'
+	<!--'.$_scriptCode.$_inlineJS.'
 	// -->
 		/*]]>*/
 	</script>';
 		} elseif ($GLOBALS['TSFE']->config['config']['removeDefaultJS']==='external')	{
-			$GLOBALS['TSFE']->content.=TSpagegen::inline2TempFile($_scriptCode, 'js');
+				// put default and inlineJS in external file
+			$GLOBALS['TSFE']->content.= TSpagegen::inline2TempFile($_scriptCode.$_inlineJS, 'js');
+		} elseif ($_inlineJS) {
+				// include only inlineJS
+			$GLOBALS['TSFE']->content.='
+	<script type="text/javascript">
+		/*<![CDATA[*/
+	<!--'.$_inlineJS.'
+	// -->
+		/*]]>*/
+	</script>';
 		}
 
-		$GLOBALS['TSFE']->content.=chr(10).implode($GLOBALS['TSFE']->additionalHeaderData,chr(10)).'
-'.$JSef[0].'
+		$GLOBALS['TSFE']->content.= chr(10).implode($GLOBALS['TSFE']->additionalHeaderData,chr(10)).'
 </head>';
 		if ($GLOBALS['TSFE']->pSetup['frameSet.'])	{
 			$fs = t3lib_div::makeInstance('tslib_frameset');
-			$GLOBALS['TSFE']->content.=$fs->make($GLOBALS['TSFE']->pSetup['frameSet.']);
+			$GLOBALS['TSFE']->content.= $fs->make($GLOBALS['TSFE']->pSetup['frameSet.']);
 			$GLOBALS['TSFE']->content.= chr(10).'<noframes>'.chr(10);
 		}
 
@@ -800,15 +857,15 @@ $GLOBALS['TSFE']->content.='
 		$GLOBALS['TSFE']->content.= chr(10).$bodyTag;
 
 
-		// Div-sections
+			// Div-sections
 		if ($GLOBALS['TSFE']->divSection)	{
 			$GLOBALS['TSFE']->content.= chr(10).$GLOBALS['TSFE']->divSection;
 		}
 
-		// Page content
+			// Page content
 		$GLOBALS['TSFE']->content.= chr(10).$pageContent;
 
-		// Ending page
+			// Ending page
 		$GLOBALS['TSFE']->content.= chr(10).'</body>';
 		if ($GLOBALS['TSFE']->pSetup['frameSet.'])	{
 			$GLOBALS['TSFE']->content.= chr(10).'</noframes>';
@@ -868,6 +925,51 @@ $GLOBALS['TSFE']->content.='
 
 		return $output;
 	}
+
+	/**
+	 * Checks if the value defined in "config.linkVars" contains an allowed value. Otherwise, return false which means the value will not be added to any links.
+	 *
+	 * @param	string		The string in which to find $needle
+	 * @param	string		The string to find in $haystack
+	 * @return	boolean		Returns true if $needle matches or is found in $haystack
+	 */
+	function isAllowedLinkVarValue($haystack,$needle)	{
+		$OK = false;
+
+		if ($needle=='int' || $needle=='integer')	{	// Integer
+
+			if (t3lib_div::testInt($haystack))	{
+				$OK = true;
+			}
+
+		} elseif (preg_match('/^\/.+\/[imsxeADSUXu]*$/', $needle))	{	// Regular expression, only "//" is allowed as delimiter
+
+			if (@preg_match($needle, $haystack))	{
+				$OK = true;
+			}
+
+		} elseif (strstr($needle,'-'))	{	// Range
+
+			if (t3lib_div::testInt($haystack))	{
+				$range = explode('-',$needle);
+				if ($range[0] <= $haystack && $range[1] >= $haystack)	{
+					$OK = true;
+				}
+			}
+
+		} elseif (strstr($needle,'|'))	{	// List
+
+			$haystack = str_replace(' ','',$haystack);	// Trim the input
+			if (strstr('|'.$needle.'|', '|'.$haystack.'|'))	{
+				$OK = true;
+			}
+
+		} elseif (!strcmp($needle,$haystack))	{	// String comparison
+			$OK = true;
+		}
+
+		return $OK;
+	}
 }
 
 if (defined('TYPO3_MODE') && $TYPO3_CONF_VARS[TYPO3_MODE]['XCLASS']['tslib/class.tslib_pagegen.php'])	{
@@ -896,8 +998,8 @@ require_once (PATH_t3lib.'class.t3lib_loaddbgroup.php');
  * @subpackage tslib
  * @see tslib_cObj::RECORDS()
  */
-class FE_loadDBGroup extends t3lib_loadDBGroup	{
-	var $fromTC = 0;		// Means the not only uid and label-field is returned, but everything
+class FE_loadDBGroup extends t3lib_loadDBGroup {
+	var $fromTC = 0;	// Means that everything is returned instead of only uid and label-field
 }
 
 // **********************************
