@@ -2,7 +2,7 @@
 /***************************************************************
 *  Copyright notice
 *
-*  (c) 1999-2004 Kasper Skaarhoj (kasperYYYY@typo3.com)
+*  (c) 1999-2008 Kasper Skaarhoj (kasperYYYY@typo3.com)
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
@@ -28,7 +28,7 @@
  * Document for viewing the online help texts, also known as TCA_DESCR.
  * See Inside TYPO3 for details.
  *
- * $Id: view_help.php,v 1.17 2004/09/14 20:33:21 typo3 Exp $
+ * $Id: view_help.php 3439 2008-03-16 19:16:51Z flyguide $
  * Revised for TYPO3 3.7 5/2004 by Kasper Skaarhoj
  * XHTML-trans compliant
  *
@@ -39,28 +39,37 @@
  *
  *
  *
- *   88: class SC_view_help
- *  111:     function init()
- *  131:     function main()
- *  160:     function printContent()
+ *   91: class local_t3lib_parsehtml extends t3lib_parsehtml
+ *  102:     function processContent($value,$dir,$conf)
+ *
+ *
+ *  122: class SC_view_help
+ *  146:     function init()
+ *  167:     function main()
+ *  198:     function printContent()
  *
  *              SECTION: Rendering main modes
- *  183:     function render_TOC()
- *  292:     function render_TOC_el($table, $tocCat, &$outputSections, &$tocArray, &$CSHkeys)
- *  324:     function render_TOC_makeTocList($tocArray)
- *  363:     function render_Table($table)
- *  420:     function render_Single($table,$field)
+ *  221:     function render_TOC()
+ *  338:     function render_TOC_el($table, $tocCat, &$outputSections, &$tocArray, &$CSHkeys)
+ *  370:     function render_TOC_makeTocList($tocArray)
+ *  409:     function render_Table($table)
+ *  469:     function render_Single($table,$field)
  *
  *              SECTION: Rendering CSH items
- *  463:     function make_seeAlso($value,$anchorTable='')
- *  512:     function printImage($images,$descr)
- *  547:     function headerLine($str,$type=0)
- *  568:     function prepareContent($str)
- *  583:     function printItem($table,$field,$anchors=0)
- *  616:     function getTableFieldNames($table,$field)
- *  639:     function getTableFieldLabel($table,$field='',$mergeToken=': ')
+ *  515:     function make_seeAlso($value,$anchorTable='')
+ *  564:     function printImage($images,$descr)
+ *  599:     function headerLine($str,$type=0)
+ *  620:     function prepareContent($str)
+ *  635:     function printItem($table,$field,$anchors=0)
+ *  668:     function getTableFieldNames($table,$field)
+ *  691:     function getTableFieldLabel($table,$field='',$mergeToken=': ')
  *
- * TOTAL FUNCTIONS: 15
+ *              SECTION: Glossary related
+ *  726:     function createGlossaryIndex()
+ *  785:     function substituteGlossaryWords($code)
+ *  801:     function substituteGlossaryWords_htmlcleaner_callback($code)
+ *
+ * TOTAL FUNCTIONS: 19
  * (This index is automatically created/updated by the extension "extdeveval")
  *
  */
@@ -69,8 +78,33 @@ require('init.php');
 require('template.php');
 $LANG->includeLLFile('EXT:lang/locallang_view_help.xml');
 require_once(PATH_t3lib.'class.t3lib_loadmodules.php');
+require_once(PATH_t3lib.'class.t3lib_parsehtml.php');
 
 
+/**
+ * Extension of the parse_html class.
+ *
+ * @author	Kasper Skaarhoj <kasperYYYY@typo3.com>
+ * @package TYPO3
+ * @subpackage core
+ */
+class local_t3lib_parsehtml extends t3lib_parsehtml {
+
+	/**
+	 * Processing content between tags for HTML_cleaner
+	 *
+	 * @param	string		The value
+	 * @param	integer		Direction, either -1 or +1. 0 (zero) means no change to input value.
+	 * @param	mixed		Not used, ignore.
+	 * @return	string		The processed value.
+	 * @access private
+	 */
+	function processContent($value,$dir,$conf)	{
+		$value = $this->pObj->substituteGlossaryWords_htmlcleaner_callback($value);
+
+		return $value;
+	}
+}
 
 
 
@@ -95,11 +129,13 @@ class SC_view_help {
 
 		// Internal, static: GPvar:
 	var $tfID;			// Table/FIeld id.
+	var $ffID;			// Flexform file/field information
 	var $back;			// Back (previous tfID)
 	var $renderALL;		// If set, then in TOC mode the FULL manual will be printed as well!
 
 		// Internal, dynamic:
 	var $content;	// Content accumulation.
+	var $glossaryWords;		// Glossary words
 
 
 
@@ -113,14 +149,20 @@ class SC_view_help {
 
 			// Setting GPvars:
 		$this->tfID = t3lib_div::_GP('tfID');
+		if (!$this->tfID) {
+			if (($this->ffID = t3lib_div::_GP('ffID'))) {
+				$this->ffID = unserialize(base64_decode($this->ffID));
+			}
+		}
 		$this->back = t3lib_div::_GP('back');
 		$this->renderALL = t3lib_div::_GP('renderALL');
 
 			// Set internal table/field to the parts of "tfID" incoming var.
 		list($this->table,$this->field) = explode('.',$this->tfID);
 
-			// limitAccess is checked if the $this->table really IS a table.
-		$this->limitAccess = isset($TCA[$this->table]) ? TRUE : FALSE;
+			// limitAccess is checked if the $this->table really IS a table (and if the user is NOT a translator who should see all!)
+		$showAllToUser = t3lib_BEfunc::isModuleSetInTBE_MODULES('txllxmltranslateM1') && $GLOBALS['BE_USER']->check('modules','txllxmltranslateM1');
+		$this->limitAccess = isset($TCA[$this->table]) ? !$showAllToUser : FALSE;
 	}
 
 	/**
@@ -137,9 +179,14 @@ class SC_view_help {
 		$this->content.= $TBE_TEMPLATE->startPage($LANG->getLL('title'));
 
 		if ($this->field=='*')	{ // If ALL fields is supposed to be shown:
+			$this->createGlossaryIndex();
 			$this->content.= $this->render_Table($this->table);
 		} elseif ($this->tfID) { // ... otherwise show only single field:
+			$this->createGlossaryIndex();
 			$this->content.= $this->render_Single($this->table,$this->field);
+		}
+		elseif (is_array($this->ffID)) {
+			$this->content.= $this->render_SingleFlex();
 		} else {	// Render Table Of Contents if nothing else:
 			$this->content.= $this->render_TOC();
 		}
@@ -233,6 +280,14 @@ class SC_view_help {
 			}
 		}
 
+			// Glossary
+		foreach($CSHkeys as $cshKey => $value)	{
+			if (t3lib_div::isFirstPartOfStr($cshKey, 'xGLOSSARY_') && !isset($TCA[$cshKey]))	{
+				$LANG->loadSingleTableDescription($cshKey);
+				$this->render_TOC_el($cshKey, 'glossary', $outputSections, $tocArray, $CSHkeys);
+			}
+		}
+
 			// Other:
 		foreach($CSHkeys as $cshKey => $value)	{
 			if (!t3lib_div::isFirstPartOfStr($cshKey, '_MOD_') && !isset($TCA[$cshKey]))	{
@@ -325,7 +380,7 @@ class SC_view_help {
 		global $LANG;
 
 			// The Various manual sections:
-		$keys = explode(',', 'core,modules,tables,extensions,other');
+		$keys = explode(',', 'core,modules,tables,extensions,glossary,other');
 
 			// Create TOC bullet list:
 		$output = '';
@@ -395,6 +450,9 @@ class SC_view_help {
 			$output.= implode('<br />',$parts);
 		}
 
+			// Substitute glossary words:
+		$output = $this->substituteGlossaryWords($output);
+
 			// TOC link:
 		if (!$this->renderALL)	{
 			$tocLink = '<p class="c-nav"><a href="view_help.php">'.$LANG->getLL('goToToc',1).'</a></p>';
@@ -428,6 +486,9 @@ class SC_view_help {
 			// Render single item:
 		$output.= $this->printItem($table,$field);
 
+			// Substitute glossary words:
+		$output = $this->substituteGlossaryWords($output);
+
 			// Link to Full table description and TOC:
 		$getLLKey = $this->limitAccess ? 'fullDescription' : 'fullDescription_module';
 		$output.= '<br />
@@ -438,13 +499,22 @@ class SC_view_help {
 	}
 
 
+	/**
+	 * Renders CSH for a single field.
+	 *
+	 * @param	string		CSH key / table name
+	 * @param	string		Sub key / field name
+	 * @return	string		HTML output
+	 */
+	function render_SingleFlex() {
+		$output = '';
 
+			// Render
+		$output.= $this->printItemFlex();
 
-
-
-
-
-
+			// Substitute glossary words:
+		return $this->substituteGlossaryWords($output);
+	}
 
 
 	/************************************
@@ -515,7 +585,7 @@ class SC_view_help {
 		$imgArray = t3lib_div::trimExplode(',', $images, 1);
 		if (count($imgArray))	{
 			$descrArray = explode(chr(10),$descr,count($imgArray));
-#debug($descrArray);
+
 			foreach($imgArray as $k => $image)	{
 				$descr = $descrArray[$k];
 
@@ -605,6 +675,35 @@ class SC_view_help {
 		return $out;
 	}
 
+	/**
+	 * Prints a single $table/$field information piece
+	 * If $anchors is set, then seeAlso references to the same table will be page-anchors, not links.
+	 *
+	 * @param	string		Table name
+	 * @param	string		Field name
+	 * @param	boolean		If anchors is to be shown.
+	 * @return	string		HTML content
+	 */
+	function printItemFlex() {
+		// Get all texts
+		foreach (explode(',', 'description,details,syntax,image,image_descr,seeAlso') as $var) {
+			// Double $ below is not a error!
+			$$var = $GLOBALS['LANG']->sL($this->ffID['cshFile'] . ':' . $this->ffID['field'] . '.' . $var);
+		}
+		// Make seeAlso references.
+		$seeAlsoRes = $this->make_seeAlso($seeAlso);
+
+			// Making item:
+		$out= $this->headerLine($this->ffID['title'], 1) .
+				$this->prepareContent($description) .
+				($details ? $this->headerLine($GLOBALS['LANG']->getLL('details').':') . $this->prepareContent($details) : '') .
+				($syntax ? $this->headerLine($GLOBALS['LANG']->getLL('syntax').':') . $this->prepareContent($syntax) : '') .
+				($image ? $this->printImage($image, $image_descr) : '') .
+				($seeAlso && $seeAlsoRes ? $this->headerLine($GLOBALS['LANG']->getLL('seeAlso').':').'<p>'.$seeAlsoRes.'</p>' : '') .
+		'<br />';
+		return $out;
+	}
+
 	/**
 	 * Returns labels for $table and $field.
 	 * If $table is "_MOD_" prefixed, the part after "_MOD_" is returned (non-tables, fx. modules)
@@ -648,6 +747,127 @@ class SC_view_help {
 
 		return $labelStr;
 	}
+
+
+
+
+
+
+
+
+
+
+
+	/******************************
+	 *
+	 * Glossary related
+	 *
+	 ******************************/
+
+	/**
+	 * Creates glossary index in $this->glossaryWords
+	 * Glossary is cached in cache_hash table and so will be updated only when cache is cleared.
+	 *
+	 * @return	void
+	 */
+	function createGlossaryIndex()	{
+		global $TCA_DESCR,$TCA,$LANG;
+
+			// Create hash string and try to retrieve glossary array:
+		$hash = md5('typo3/view_help.php:glossary');
+ 		list($this->glossaryWords,$this->substWords) = unserialize(t3lib_BEfunc::getHash($hash));
+
+			// Generate glossary words if not found:
+		if (!is_array($this->glossaryWords)) {
+
+				// Initialize:
+			$this->glossaryWords = array();
+			$this->substWords = array();
+			$CSHkeys = array_flip(array_keys($TCA_DESCR));
+
+				// Glossary
+			foreach($CSHkeys as $cshKey => $value)	{
+				if (t3lib_div::isFirstPartOfStr($cshKey, 'xGLOSSARY_') && !isset($TCA[$cshKey]))	{
+					$LANG->loadSingleTableDescription($cshKey);
+
+					if (is_array($TCA_DESCR[$cshKey]['columns']))	{
+
+							// Traverse table columns as listed in TCA_DESCR
+						reset($TCA_DESCR[$cshKey]['columns']);
+						while(list($field,$data) = each($TCA_DESCR[$cshKey]['columns']))	{
+							if ($field)	{
+								$this->glossaryWords[$cshKey.'.'.$field] = array(
+									'title' => trim($data['alttitle'] ? $data['alttitle'] : $cshKey),
+									'description' =>  str_replace('%22','%23%23%23', rawurlencode($data['description'])),
+								);
+							}
+						}
+					}
+				}
+			}
+
+				// First, create unique list of words:
+			foreach($this->glossaryWords as $key => $value)	{
+				$word = strtolower($value['title']);	// Making word lowercase in order to filter out same words in different cases.
+
+				if ($word!=='')	{
+					$this->substWords[$word] = $value;
+					$this->substWords[$word]['key'] = $key;
+				}
+			}
+
+			krsort($this->substWords);
+
+			t3lib_BEfunc::storeHash($hash,serialize(array($this->glossaryWords,$this->substWords)),'Glossary');
+		}
+	}
+
+	/**
+	 * Processing of all non-HTML content in the output
+	 * Will be done by a call-back to ->substituteGlossaryWords_htmlcleaner_callback()
+	 *
+	 * @param	string		Input HTML code
+	 * @return	string		Output HTML code
+	 */
+	function substituteGlossaryWords($code) {
+		$htmlParser = t3lib_div::makeInstance('local_t3lib_parsehtml');
+		$htmlParser->pObj = &$this;
+		$code = $htmlParser->HTMLcleaner($code, array(), 1);
+
+		return $code;
+	}
+
+	/**
+	 * Substituting glossary words in the CSH
+	 * (This is a call-back function from "class local_t3lib_parsehtml extends t3lib_parsehtml", see top of this script)
+	 *
+	 * @param	string		Input HTML string
+	 * @return	string		HTML with substituted words in.
+	 * @coauthor	alex widschwendter, media.res kommunikationsloesungen
+	 */
+	function substituteGlossaryWords_htmlcleaner_callback($code)	{
+		if (is_array($this->substWords) && count($this->substWords) && strlen(trim($code)))	{
+
+				// Substitute words:
+			foreach($this->substWords as $wordKey => $wordSet)	{
+					// quoteMeta used so special chars (which should not occur though) in words will not break the regex. Seemed to work (- kasper)
+				$parts = preg_split('/( |[\(])('.quoteMeta($wordSet['title']).')([\.\!\)\?\:\,]+| )/i', ' '.$code.' ', 2, PREG_SPLIT_DELIM_CAPTURE);
+				if (count($parts) == 5)	{
+					$parts[2] = '<a class="glossary-term" href="'.htmlspecialchars('view_help.php?tfID='.rawurlencode($wordSet['key']).'&back='.$this->tfID).'" title="'.rawurlencode(htmlspecialchars(t3lib_div::fixed_lgd_cs(rawurldecode($wordSet['description']),80))).'">'.
+								htmlspecialchars($parts[2]).
+								'</a>';
+					$code = substr(implode('',$parts),1,-1);
+
+						// Disable entry so it doesn't get used next time:
+					unset($this->substWords[$wordKey]);
+				}
+			}
+			$code = str_replace('###', '&quot;',rawurldecode($code));
+		}
+
+		return $code;
+	}
+
 }
 
 
